%% demo of the power of CVX

load 2moons
Xi = x;
yi = y;
[n,p] = size(Xi);

figure(1)
set(gcf,'Color',[1,1,1])
plot(Xi(find(yi==1),1),Xi(find(yi==1),2),'o')
hold on
plot(Xi(find(yi==-1),1),Xi(find(yi==-1),2),'xr')
Xi1 = Xi(find(yi==1),:);
Xi2 = Xi(find(yi==-1),:);

%%
kernel = 'gaussian';

kerneloption = .3;
lam = .05;

kerneloption = 1;
lam = .5;

% kerneloption = 2.5;
% lam = .5;
% lam = 500;

K = svmkernel(Xi,kernel,kerneloption);
G = (yi*yi').*K;
sG = sqrtm(G);

nx = diag(G);
e = ones(n,1);

cvx_quiet(true)
fprintf('        CVX: ')
tic
cvx_begin
   cvx_precision best
   variables a(n) 
   dual variables eq c0 cc 
   minimize(  norm(sG*a)  - a'*(yi.*nx) )
%   minimize(  a'*G*a  )
   subject to
      eq  : a'*yi == 1;
     c0 :  a >=0;
     cc :  a <=lam;
cvx_end
toc

pos = find(a > 0.00001);
am = a(pos);
lambda = min(1  - 2*K(pos,pos)*(yi(pos).*a(pos)));

%% display

[xtest1 xtest2]  = meshgrid([-1:.01:1]*2+.5,[-1:0.01:1]*1.+.5); 
nn = length(xtest1); 
Xgrid = [reshape(xtest1 ,nn*nn,1) reshape(xtest2 ,nn*nn,1)]; 
%--------------  Evaluating the decision function
Kgrid = svmkernel(Xgrid,kernel,kerneloption,Xi(pos,:));
ypred = 1  - 2*Kgrid*(yi(pos).*am) - lambda;
ypred = reshape(ypred,nn,nn); 

figure(1)
clf
set(gcf,'Color',[1,1,1])
hold on
contourf(xtest1,xtest2,ypred,50); shading flat; 
contour(xtest1,xtest2,ypred,[-10 0],'r');
[cc,hh]=contour(xtest1,xtest2,ypred,[-1  0],'k','LineWidth',2);
set(gca,'FontSize',24,'FontName','Arial','XTick',[ ],'YTick',[ ],'Box','on');  


plot(Xi2(:,1),Xi2(:,2),'xw','LineWidth',2)
h2 = plot(Xi1(:,1),Xi1(:,2),'+r','LineWidth',2);
%plot(Xi(:,1),Xi(:,2),'+w','LineWidth',2);
%pos = find(a>eps^(1/3));
h1 = plot(Xi(pos,1),Xi(pos,2),'ob'); 
set(h1,'LineWidth',1,...
          'MarkerEdgeColor','c',...
          'MarkerSize',15);
hold off
%axis([-3 3 -1.5 2.5]);


%%  L2 C-SVM

kerneloption = .35;
K = svmkernel(Xi,kernel,kerneloption);
G = (yi*yi').*K;
sG = sqrtm(G);

C = 10000000000;

cvx_quiet(true)
fprintf('        CVX: ')
tic
cvx_begin
   cvx_precision best
   variables a(n) 
   dual variables eq c0 cc 
%   minimize(  norm(sG*a)  - 2*a'*e )
   minimize( .5* a'*G*a  - a'*e )
   subject to
      eq  : a'*yi == 0;
     c0 :  a >=0;
     cc :  a <=C;
cvx_end
toc

%[am, lambda, pos] = monqp(G,(e),yi,0,C,0,0);

pos = find(a > 0.00001);
am = a(pos);

%% display

[xtest1 xtest2]  = meshgrid([-1:.01:1]*2+.5,[-1:0.01:1]*1.+.5); 
nn = length(xtest1); 
Xgrid = [reshape(xtest1 ,nn*nn,1) reshape(xtest2 ,nn*nn,1)]; 
%--------------  Evaluating the decision function
Kgrid = svmkernel(Xgrid,kernel,kerneloption,Xi(pos,:));
ypred = Kgrid*(yi(pos).*am) + eq;
ypred = reshape(ypred,nn,nn); 

figure(2)
clf
set(gcf,'Color',[1,1,1])
hold on
contourf(xtest1,xtest2,ypred,50); shading flat; 
contour(xtest1,xtest2,ypred,[-10 0],'r');
[cc,hh]=contour(xtest1,xtest2,ypred,[-1   1],'k:','LineWidth',1);
[cc,hh]=contour(xtest1,xtest2,ypred,[-1000 0 ],'k','LineWidth',2);
set(gca,'FontSize',24,'FontName','Arial','XTick',[ ],'YTick',[ ],'Box','on');  


plot(Xi2(:,1),Xi2(:,2),'xw','LineWidth',2)
h2 = plot(Xi1(:,1),Xi1(:,2),'+r','LineWidth',2);
%plot(Xi(:,1),Xi(:,2),'+w','LineWidth',2);
%pos = find(a>eps^(1/3));
h1 = plot(Xi(pos,1),Xi(pos,2),'ob'); 
set(h1,'LineWidth',1,...
          'MarkerEdgeColor','c',...
          'MarkerSize',15);
hold off
%axis([-3 3 -1.5 2.5]);
%title('The two moons  and 2 class SVM')

%% SVDD

kerneloption = .6;
K = svmkernel(Xi1,kernel,kerneloption);
sK = sqrtm(K);
n1 = length(Xi1);
e1 = ones(n1,1);

lam = .2;

cvx_quiet(true)
fprintf('        CVX: ')
tic
cvx_begin
   cvx_precision best
   variables a(n1) 
   dual variables eq c0 cc 
   minimize(  norm(sK*a)  - a'*diag(K) )
%   minimize(  a'*G*a  - a'*e )
   subject to
      eq  : a'*e1 == 1;
     c0 :  a >=0;
     cc :  a <=lam;
cvx_end
toc

%[am, lambda, pos] = monqp(G,(e),yi,0,C,0,0);

pos = find(a > 0.00001);
am = a(pos);
lambda = min(1  - 2*K(pos,pos)*(a(pos)));

%% display

[xtest1 xtest2]  = meshgrid([-1:.01:1]*2+.5,[-1:0.01:1]*1.+.5); 
nn = length(xtest1); 
Xgrid = [reshape(xtest1 ,nn*nn,1) reshape(xtest2 ,nn*nn,1)]; 
%--------------  Evaluating the decision function
Kgrid = svmkernel(Xgrid,kernel,kerneloption,Xi1(pos,:));
ypred = 1  - 2*Kgrid*am - lambda;
ypred = reshape(ypred,nn,nn); 

figure(3)
clf
set(gcf,'Color',[1,1,1])
hold on
contourf(xtest1,xtest2,ypred,50); shading flat; 
contour(xtest1,xtest2,ypred,[-10 0],'r');
[cc,hh]=contour(xtest1,xtest2,ypred,[-1   1],'k:','LineWidth',1);
[cc,hh]=contour(xtest1,xtest2,ypred,[-1000 0 ],'k','LineWidth',2);
set(gca,'FontSize',24,'FontName','Arial','XTick',[ ],'YTick',[ ],'Box','on');  


plot(Xi2(:,1),Xi2(:,2),'xw','LineWidth',2)
h2 = plot(Xi1(:,1),Xi1(:,2),'+r','LineWidth',2);
%plot(Xi(:,1),Xi(:,2),'+w','LineWidth',2);
%pos = find(a>eps^(1/3));
h1 = plot(Xi1(pos,1),Xi1(pos,2),'ob'); 
set(h1,'LineWidth',1,...
          'MarkerEdgeColor','c',...
          'MarkerSize',15);
hold off
%axis([-3 3 -1.5 2.5]);
%title('The two moons  and 1 class SVDD')

%% 2 classes SVDD